#!/usr/bin/python
#####################################################################
#
# Copyright (C) 2019 Oracle.  All rights reserved.
#
#####################################################################
import sys
import subprocess

olv_unprocessed_file = "/tmp/ovirt_unprocessed.txt"
olv_versions_file = "/tmp/ovirt_versions.txt"
olv_versions_python_file = "/tmp/ovirt_versions.py"

print "Collecting Oracle Linux Virtualization product versions ...\n"

isengine = subprocess.call(
    "yum list installed ovirt-engine",
    stdout=subprocess.PIPE,
    stderr=subprocess.STDOUT,
    shell=True)
isvdsm = subprocess.call(
    "yum list installed vdsm",
    stdout=subprocess.PIPE,
    stderr=subprocess.STDOUT,
    shell=True)

#Successful 0 return means the command succeeded and the package is installed.
if not isengine:
    print "Detected system has ovirt-engine installed.\n"
    rpmcommand = (
        "yum list installed |"
        "grep ovirt && "
        "rpm -qR ovirt-engine |"
        "xargs yum list installed ovirt-engine"
        )

elif not isvdsm:
    print "Detected system has vdsm installed.\n"
    rpmcommand = (
        "yum list installed |"
        "grep ovirt && "
        "rpm -qR vdsm ovirt-host |"
        "xargs yum list installed vdsm ovirt-host"
        )

else:
    print "This system does not have ovirt-engine or vdsm installed.\n"
    sys.exit(1)

olv_versions = open(olv_versions_file, "w+")
olv_versions_python = open(olv_versions_python_file, "w+")
olv_unprocessed = open(olv_unprocessed_file, "w+")

output = subprocess.check_output(rpmcommand, shell=True)

numprocessed = 0
numunprocessed = 0
rpmdict = {}
allrpmpackages = output.splitlines()
for rpmpackage in allrpmpackages:
    #print "processing rpm package = " + rpmpackage
    rpmpackage = rpmpackage.split()
    if len(rpmpackage) != 3:
        #print "Skipping %s -- failure to parse." % rpmpackage
        olv_unprocessed.write("%s\r\n" % rpmpackage)
        numunprocessed += 1
    else:
        #TODO check for duplicate rpms ?
        rpmdict.setdefault(rpmpackage[0], {})['version'] = rpmpackage[1]
        rpmdict[rpmpackage[0]]['sourcerepo'] = rpmpackage[2]
        numprocessed += 1

#Write the header of the versions file
if not isengine:
    olv_versions.write(
        "Package list for ovirt-engine version %s\n"
        % rpmdict['ovirt-engine.noarch']['version'])

    summary = (
        "for ovirt-engine version %s"
        % rpmdict['ovirt-engine.noarch']['version']
        )

if not isvdsm:
    olv_versions.write(
        "Package list for vdsm version %s"
        % rpmdict['vdsm.x86_64']['version'])
    summary = (
        "for vdsm version %s"
        % rpmdict['vdsm.x86_64']['version']
        )
    if 'ovirt-host.x86_64' in rpmdict:
        olv_versions.write(
            " and ovirt-host %s"
            % rpmdict['ovirt-host.x86_64']['version'])
        summary = (
            "%s and ovirt-host %s"
            % (summary, rpmdict['ovirt-host.x86_64']['version'])
            )
    olv_versions.write("\n")

#Need to write the versions file separately as we want this to be sorted.
for k, v in sorted(rpmdict.items()):
    olv_versions.write(
        "%s,%s,%s\n"
        % (k, rpmdict[k]['version'], rpmdict[k]['sourcerepo']))

#Write the python dictionary to file (possibly useful in diffing)
olv_versions_python.write("%s" % rpmdict)

olv_versions.close()
olv_versions_python.close()
olv_unprocessed.close()

print "Processed %d rpm packages %s\n" % (len(rpmdict), summary)
print "See %s for details." % (olv_versions_file)
